/* $NetBSD$ */

/* NetBSD: powerpc/ibm4xx/dev/ibm405gp.c,v 1.4 2005/12/11 12:18:42 christos */
/* NetBSD: powerpc/ibm4xx/pci/pchb.c,v 1.5 2005/12/11 12:18:43 christos */
/* NetBSD: powerpc/ibm4xx/pci/pci_machdep.c,v 1.4 2006/03/29 17:50:33 shige */

/*-
 * Copyright (c) 1996 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Jason R. Thorpe.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *        This product includes software developed by the NetBSD
 *        Foundation, Inc. and its contributors.
 * 4. Neither the name of The NetBSD Foundation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Copyright 2001 Wasabi Systems, Inc.
 * All rights reserved.
 *
 * Written by Eduardo Horvath and Simon Burge for Wasabi Systems, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed for the NetBSD Project by
 *      Wasabi Systems, Inc.
 * 4. The name of Wasabi Systems, Inc. may not be used to endorse
 *    or promote products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY WASABI SYSTEMS, INC. ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WASABI SYSTEMS, INC
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Copyright (c) 1996 Christopher G. Demetriou.  All rights reserved.
 * Copyright (c) 1994 Charles M. Hannum.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Charles M. Hannum.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include "pci.h"
#include "opt_pci.h"

#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/device.h>
#include <sys/extent.h>
#include <sys/malloc.h>

#define _IBM4XX_BUS_DMA_PRIVATE

#include <machine/bus.h>
#include <machine/intr.h>

#include <powerpc/ibm4xx/dev/plbvar.h>
#include <powerpc/ibm4xx/dev/pchbvar.h>
#include <powerpc/ibm4xx/dev/pcicreg.h>

#include <dev/pci/pcivar.h>
#include <dev/pci/pcireg.h>
#include <dev/pci/pcidevs.h>
#include <dev/pci/pciconf.h>


struct pchb_softc {
	struct device			sc_dev;
	struct ibm4xx_pci_chipset	sc_pc;

	bus_addr_t			sc_lowmem_base;
	bus_addr_t			sc_highmem_base;
	bus_space_tag_t			sc_pchb_iot;
	bus_space_tag_t			sc_pchb_memt;

	/* PCI Configuration */
	bus_space_tag_t			sc_pcic_iot;
	bus_space_handle_t		sc_pcic_ioh;

	/* PCI Local Configuration */
	bus_space_tag_t			sc_pcil_iot;
	bus_space_handle_t		sc_pcil_ioh;
};

static int	pchbmatch(struct device *, struct cfdata *, void *);
static void	pchbattach(struct device *, struct device *, void *);
static int	pchbprint(void *, const char *);

CFATTACH_DECL(pchb, sizeof(struct pchb_softc),
	pchbmatch, pchbattach, NULL, NULL);

static void	setup_pci(struct pchb_softc *);
#ifdef PCI_CONFIGURE_VERBOSE
static void	show_pci_map(struct pchb_softc *);
#endif

static int pcifound = 0;
static char ex_storage[EXTENT_FIXED_STORAGE_SIZE(1)]
	__attribute__((aligned(8)));


static int
pchbmatch(struct device *parent, struct cfdata *cf, void *aux)
{
	struct plb_attach_args *paa = aux;

	/* match only pchb devices */
	if (strcmp(paa->plb_name, cf->cf_name) != 0)
		return 0;

	if (pcifound)
		return 0;

	return 1;
}

static void
pchbattach(struct device *parent, struct device *self, void *aux)
{
	struct pchb_softc *sc = (struct pchb_softc *)self;
	struct plb_attach_args *paa = aux;
	struct ibm4xx_pci_config *pci = (struct ibm4xx_pci_config *)(paa->arg);
	struct pcibus_attach_args pba;
	pcitag_t tag; 
	int class, id;
	char devinfo[256];

#ifdef PCI_CONFIGURE_VERBOSE
	extern int pci_conf_debug;

	pci_conf_debug = 1;
#endif

	sc->sc_pc.rootnode = 0;
	sc->sc_pc.sc = sc;
	sc->sc_lowmem_base = pci->pci_lowmem_base;
	sc->sc_highmem_base = pci->pci_highmem_base;
	sc->sc_pchb_iot = &(pci->pchb_io_bust);
	sc->sc_pchb_memt = &(pci->pchb_mem_bust);
	sc->sc_pcic_iot = &(pci->pcic_bust);
	sc->sc_pcil_iot = &(pci->pcil_bust);

#ifdef PCI_CONFIGURE_VERBOSE
	aprint_normal("\n");
	aprint_normal("PCI Low  Mem: 0x%08x\n", pci->pci_lowmem_base);
	aprint_normal("PCI High Mem: 0x%08x\n", pci->pci_highmem_base);

	aprint_normal("PCI-HOST IO:\n");
	aprint_normal("      offset: 0x%08x\n", pci->pchb_io_bust.pbs_offset);
	aprint_normal("        base: 0x%08x\n", pci->pchb_io_bust.pbs_base);
	aprint_normal("       limit: 0x%08x\n", pci->pchb_io_bust.pbs_limit);

	aprint_normal("PCI-HOST MEM:\n");
	aprint_normal("      offset: 0x%08x\n", pci->pchb_mem_bust.pbs_offset);
	aprint_normal("        base: 0x%08x\n", pci->pchb_mem_bust.pbs_base);
	aprint_normal("       limit: 0x%08x\n", pci->pchb_mem_bust.pbs_limit);

	aprint_normal("PCI Configuration:\n");
	aprint_normal("      offset: 0x%08x\n", pci->pcic_bust.pbs_offset);
	aprint_normal("        base: 0x%08x\n", pci->pcic_bust.pbs_base);
	aprint_normal("       limit: 0x%08x\n", pci->pcic_bust.pbs_limit);

	aprint_normal("PCI Local Configuration:\n");
	aprint_normal("      offset: 0x%08x\n", pci->pcil_bust.pbs_offset);
	aprint_normal("        base: 0x%08x\n", pci->pcil_bust.pbs_base);
	aprint_normal("       limit: 0x%08x\n", pci->pcil_bust.pbs_limit);
#endif

	if (bus_space_init(&(pci->pcic_bust), "pcicfg", NULL, 0)
		|| bus_space_map(sc->sc_pcic_iot, sc->sc_pcic_iot->pbs_base,
		(sc->sc_pcic_iot->pbs_limit - sc->sc_pcic_iot->pbs_base),
		0, &(sc->sc_pcic_ioh)) ) {
		aprint_normal(": unable to map PCI configuration registers\n");
		return;
	}

	if (bus_space_init(&(pci->pcil_bust), "pcilocalcfg", ex_storage,
			sizeof(ex_storage))
		|| bus_space_map(sc->sc_pcil_iot, sc->sc_pcil_iot->pbs_base,
		(sc->sc_pcil_iot->pbs_limit - sc->sc_pcil_iot->pbs_base),
		0, &(sc->sc_pcil_ioh)) ) {
		aprint_normal
			(": unable to map PCI Local configuration registers\n");
		return;
	}

	tag = pci_make_tag(&(sc->sc_pc), 0, 0, 0);

	class = pci_conf_read(&(sc->sc_pc), tag, PCI_CLASS_REG);
	id = pci_conf_read(&(sc->sc_pc), tag, PCI_ID_REG);

	printf("\n");
	pcifound++;

	/*
	 * All we do is print out a description.  Eventually, we
	 * might want to add code that does something that's
	 * possibly chipset-specific.
	 */

	pci_devinfo(id, class, 0, devinfo, sizeof(devinfo));
	printf("%s: %s (rev. 0x%02x)\n", self->dv_xname, devinfo,
		PCI_REVISION(class));

	setup_pci(sc);
#ifdef PCI_CONFIGURE_VERBOSE
	show_pci_map(sc);
#endif

	if (bus_space_init(&(pci->pchb_mem_bust), "pchbmem", NULL, 0))
		panic("pchbattach: can't init MEM tag");
	if (bus_space_init(&(pci->pchb_io_bust), "pchbio", NULL, 0))
		panic("pchbattach: can't init IO tag");

#ifdef PCI_NETBSD_CONFIGURE
	sc->sc_pc.memext = extent_create("pcimem",
		sc->sc_pchb_memt->pbs_base, sc->sc_pchb_memt->pbs_limit,
		M_DEVBUF, NULL, 0, EX_NOWAIT);
	sc->sc_pc.ioext = extent_create("pciio",
		sc->sc_pchb_iot->pbs_base, sc->sc_pchb_iot->pbs_limit,
		M_DEVBUF, NULL, 0, EX_NOWAIT);

	/* XXX 32 is magic number */
	pci_configure_bus(&(sc->sc_pc), sc->sc_pc.ioext, sc->sc_pc.memext,
		NULL, 0, 32);
#endif /* PCI_NETBSD_CONFIGURE */

#ifdef PCI_CONFIGURE_VERBOSE
	printf("running config_found PCI\n");
#endif

	pba.pba_iot = sc->sc_pchb_iot;
	pba.pba_memt = sc->sc_pchb_memt;
	pba.pba_dmat = paa->plb_dmat;
	pba.pba_dmat64 = NULL;
	pba.pba_pc = &(sc->sc_pc);
	pba.pba_bus = 0;
	pba.pba_bridgetag = NULL;
	pba.pba_flags = PCI_FLAGS_MEM_ENABLED | PCI_FLAGS_IO_ENABLED;

	config_found_ia(self, "pcibus", &pba, pchbprint);
}


static int
pchbprint(void *aux, const char *p)
{

	if (p == NULL)
		return (UNCONF);
	return QUIET;
}

/*
 * Setup proper Local<->PCI mapping
 * PCI memory window: 256M @ PCI0MEMBASE with direct memory translation
 */
static void
setup_pci(struct pchb_softc *sc)
{
	pcitag_t tag;

#define pcil_bus_space_write_4(c, o, v)	\
	bus_space_write_4((c)->sc_pcil_iot, (c)->sc_pcil_ioh, o, v);

	/* Disable all three memory mappers */
	pcil_bus_space_write_4(sc, PCIL_PMM0MA, 0x00000000); /* disabled */
	pcil_bus_space_write_4(sc, PCIL_PMM1MA, 0x00000000); /* disabled */
	pcil_bus_space_write_4(sc, PCIL_PMM2MA, 0x00000000); /* disabled */
	pcil_bus_space_write_4(sc, PCIL_PTM1MS, 0x00000000);
					/* Can't really disable PTM1 */
	pcil_bus_space_write_4(sc, PCIL_PTM2MS, 0x00000000); /* disabled */

	/* Setup memory map #0 */
	pcil_bus_space_write_4(sc, PCIL_PMM0MA, 0xf0000001);
					/* 256M non-prefetchable, enabled */

	pcil_bus_space_write_4(sc, PCIL_PMM0LA, sc->sc_lowmem_base);
	pcil_bus_space_write_4(sc, PCIL_PMM0PCILA, sc->sc_lowmem_base);
	pcil_bus_space_write_4(sc, PCIL_PMM0PCIHA, sc->sc_highmem_base);

	/* Configure PCI bridge */
	tag = pci_make_tag(&(sc->sc_pc), 0, 0, 0);
  
	pcil_bus_space_write_4(sc, PCIL_PTM1MS, 0xf0000001); /* Enable PTM1 */
	pcil_bus_space_write_4(sc, PCIL_PTM1LA, 0);

	/* Set up proper PCI->Local address base.  Always enabled */
	pci_conf_write(&(sc->sc_pc), tag, PCIC_PTM1BAR, 0);
	pci_conf_write(&(sc->sc_pc), tag, PCIC_PTM2BAR, 0);
}

#ifdef PCI_CONFIGURE_VERBOSE
static void
show_pci_map(struct pchb_softc *sc)
{
	paddr_t la, lm, pl, ph;
	pcitag_t tag;

#define pcil_bus_space_read_4(c, o)	\
	bus_space_read_4((c)->sc_pcil_iot, (c)->sc_pcil_ioh, o);

	printf("Local -> PCI map\n");

	la = pcil_bus_space_read_4(sc, PCIL_PMM0LA);
	lm = pcil_bus_space_read_4(sc, PCIL_PMM0MA);
	pl = pcil_bus_space_read_4(sc, PCIL_PMM0PCILA);
	ph = pcil_bus_space_read_4(sc, PCIL_PMM0PCIHA);
	printf("0: %08lx,%08lx -> %08lx%08lx %sprefetchable, %s\n",
		la, lm, ph, pl,
		(lm & 2) ? "":"not ",
		(lm & 1) ? "enabled":"disabled");

	la = pcil_bus_space_read_4(sc, PCIL_PMM1LA);
	lm = pcil_bus_space_read_4(sc, PCIL_PMM1MA);
	pl = pcil_bus_space_read_4(sc, PCIL_PMM1PCILA);
	ph = pcil_bus_space_read_4(sc, PCIL_PMM1PCIHA);
	printf("1: %08lx,%08lx -> %08lx%08lx %sprefetchable, %s\n",
		la, lm, ph, pl,
		(lm & 2) ? "":"not ",
		(lm & 1) ? "enabled":"disabled");

	la = pcil_bus_space_read_4(sc, PCIL_PMM2LA);
	lm = pcil_bus_space_read_4(sc, PCIL_PMM2MA);
	pl = pcil_bus_space_read_4(sc, PCIL_PMM2PCILA);
	ph = pcil_bus_space_read_4(sc, PCIL_PMM2PCIHA);
	printf("2: %08lx,%08lx -> %08lx%08lx %sprefetchable, %s\n",
		la, lm, ph, pl,
		(lm & 2) ? "":"not ",
		(lm & 1) ? "enabled":"disabled");

	printf("PCI -> Local map\n");
	tag = pci_make_tag(&(sc->sc_pc), 0, 0, 0);

	pl = pci_conf_read(&(sc->sc_pc), tag, PCIC_PTM1BAR);
	la = pcil_bus_space_read_4(sc, PCIL_PTM1LA);
	lm = pcil_bus_space_read_4(sc, PCIL_PTM1MS);
	printf("1: %08lx -> %08lx,%08lx %s\n", pl, la, lm,
	    (lm & 1)?"enabled":"disabled");

	pl = pci_conf_read(&(sc->sc_pc), tag, PCIC_PTM2BAR);
	la = pcil_bus_space_read_4(sc, PCIL_PTM2LA);
	lm = pcil_bus_space_read_4(sc, PCIL_PTM2MS);
	printf("2: %08lx -> %08lx,%08lx %s\n", pl, la, lm,
	    (lm & 1)?"enabled":"disabled");
}
#endif

/*
 * Machine-specific functions for PCI autoconfiguration.
 *
 * On PCs, there are two methods of generating PCI configuration cycles.
 * We try to detect the appropriate mechanism for this machine and set
 * up a few function pointers to access the correct method directly.
 *
 * The configuration method can be hard-coded in the config file by
 * using `options PCI_CONF_MODE=N', where `N' is the configuration mode
 * as defined section 3.6.4.1, `Generating Configuration Cycles'.
 */

void
pci_attach_hook(struct device *parent, struct device *self,
		struct pcibus_attach_args *pba)
{
#if 0
	struct pchb_softc *sc = (struct pchb_softc *)(pba->pba_pc.sc);

#ifdef PCI_CONFIGURE_VERBOSE
	printf("pci_attach_hook\n");
	show_pci_map(sc);
#endif
	setup_pci(sc);

#ifdef PCI_CONFIGURE_VERBOSE
	show_pci_map(sc);
#endif
#endif
}

int
pci_bus_maxdevs(pci_chipset_tag_t pc, int busno)
{

	/*
	 * Bus number is irrelevant.  Configuration Mechanism 1 is in
	 * use, can have devices 0-32 (i.e. the `normal' range).
	 */
	return 32;
}

pcitag_t
pci_make_tag(pci_chipset_tag_t pc, int bus, int device, int function)
{
	pcitag_t tag;

	if (bus >= 256 || device >= 32 || function >= 8)
		panic("pci_make_tag: bad request");

	/* XXX magic number */
	tag = 0x80000000 | (bus << 16) | (device << 11) | (function << 8);

	return tag;
}

void
pci_decompose_tag(pci_chipset_tag_t pc, pcitag_t tag, int *bp, int *dp, int *fp)
{

	if (bp != NULL)
		*bp = (tag >> 16) & 0xff;
	if (dp != NULL)
		*dp = (tag >> 11) & 0x1f;
	if (fp != NULL)
		*fp = (tag >> 8) & 0x07;
}

pcireg_t
pci_conf_read(pci_chipset_tag_t pc, pcitag_t tag, int reg)
{
	struct pchb_softc *sc = (struct pchb_softc *)(pc->sc);
	pcireg_t data;

	/* 405GT BIOS disables interrupts here. Should we? --Art */
	bus_space_write_4(sc->sc_pcic_iot, sc->sc_pcic_ioh,
		PCIC_CFGADDR, tag | reg);
	data = bus_space_read_4(sc->sc_pcic_iot, sc->sc_pcic_ioh, PCIC_CFGDATA);

	/* 405GP pass2 errata #6 */
	bus_space_write_4(sc->sc_pcic_iot, sc->sc_pcic_ioh, PCIC_CFGADDR, 0);

	return data;
}

void
pci_conf_write(pci_chipset_tag_t pc, pcitag_t tag, int reg, pcireg_t data)
{
	struct pchb_softc *sc = (struct pchb_softc *)(pc->sc);

	bus_space_write_4(sc->sc_pcic_iot, sc->sc_pcic_ioh,
		PCIC_CFGADDR, tag | reg);
	bus_space_write_4(sc->sc_pcic_iot, sc->sc_pcic_ioh, PCIC_CFGDATA, data);

	/* 405GP pass2 errata #6 */
	bus_space_write_4(sc->sc_pcic_iot, sc->sc_pcic_ioh, PCIC_CFGADDR, 0);
}

const char *
pci_intr_string(pci_chipset_tag_t pc, pci_intr_handle_t ih)
{
	static char irqstr[8];	/* 4 + 2 + NUL + sanity */

	if (ih == 0 || ih >= ICU_LEN)
		panic("pci_intr_string: bogus handle 0x%x", ih);

	sprintf(irqstr, "irq %d", ih);
	return (irqstr);
	
}

const struct evcnt *
pci_intr_evcnt(pci_chipset_tag_t pc, pci_intr_handle_t ih)
{

	/* XXX for now, no evcnt parent reported */
	return NULL;
}

void *
pci_intr_establish(pci_chipset_tag_t pc, pci_intr_handle_t ih, int level,
		   int (*func)(void *), void *arg)
{

	if (ih == 0 || ih >= ICU_LEN)
		panic("pci_intr_establish: bogus handle 0x%x", ih);

	return intr_establish(ih, IST_LEVEL, level, func, arg);
}

void
pci_intr_disestablish(pci_chipset_tag_t pc, void *cookie)
{

	intr_disestablish(cookie);
}

/* Avoid overconfiguration */
int
pci_conf_hook(pci_chipset_tag_t pc, int bus, int dev, int func, pcireg_t id)
{

	if ((PCI_VENDOR(id) == PCI_VENDOR_IBM
		&& PCI_PRODUCT(id) == PCI_PRODUCT_IBM_405GP)
		||
	    (PCI_VENDOR(id) == PCI_VENDOR_INTEL
		&& PCI_PRODUCT(id) == PCI_PRODUCT_INTEL_80960_RP)) {
		/* Don't configure the bridge and PCI probe. */
		return 0;
	}
	return PCI_CONF_DEFAULT;
}
