/* $NetBSD$ */

/*
 * Copyright (c) 2006 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed
 * to The NetBSD Foundation, Inc.
 * by Shigeyuki Fukushima.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the NetBSD
 *      Foundation, Inc. and its contributors.
 * 4. Neither the name of The NetBSD Foundation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>

#include <machine/cpu.h>

#include <powerpc/ibm4xx/ibm405gpr.h>
#include <powerpc/ibm4xx/ibm4xx.h>
#include <powerpc/ibm4xx/dev/plbvar.h>
#include <powerpc/ibm4xx/dev/opbvar.h>
#include <powerpc/ibm4xx/dev/pchbvar.h>

struct opb_dev ibm405gpr_opb_devices[] = {
	{ "com",     0, UART0_BASE, {  0, -1, -1, -1, -1, -1, -1, -1 }, NULL },
	{ "com",     1, UART1_BASE, {  1, -1, -1, -1, -1, -1, -1, -1 }, NULL },
	{ "emac",    0, EMAC0_BASE, {  9, 10, 11, 12, 13, 14, 15, -1 }, NULL },
	{ "opbgpio", 0, GPIO0_BASE, { -1, -1, -1, -1, -1, -1, -1, -1 }, NULL },
	{ "gpiic",   0, IIC0_BASE,  {  2, -1, -1, -1, -1, -1, -1, -1 }, NULL },
	{ "wdog",    0, -1,         { -1, -1, -1, -1, -1, -1, -1, -1 }, NULL },
	{ NULL }
};

struct opb_bus_space ibm405gpr_opb_bus_space = {
	UART0_BASE,
	UART0_BASE + 0xffff,
};

struct ibm4xx_pci_config ibm405gpr_pci_config = {
	/* pci_lowmem_base */
	PCI_MEM_START,
	/* pci_highmem_base */
	0,
	/* pchb_io_bust */
	{
		_BUS_SPACE_LITTLE_ENDIAN | _BUS_SPACE_IO_TYPE,
		PLB_PCI_IO_START,			/* offset */
		PCI_PCI_IO_START,			/* extent base */
		PCI_PCI_IO_START + PCI_PCI_IO_SIZE	/* extent limit */
	},
	/* pchb_mem_bust */
	{
		_BUS_SPACE_LITTLE_ENDIAN | _BUS_SPACE_MEM_TYPE,
		0x00000000,				/* offset */
		PCI_MEM_START,				/* extent base */
		PCI_MEM_START + 0x1fffffff		/* extent limit */
	},
	/* pcic_bust */
	{
		_BUS_SPACE_LITTLE_ENDIAN | _BUS_SPACE_MEM_TYPE,
		0x00000000,				/* offset */
		PCIC0_BASE,				/* extent base */
		PCIC0_BASE + PCIC0_SIZE			/* extent limit */
	},
	/* pcil_bust */
	{
		_BUS_SPACE_LITTLE_ENDIAN|_BUS_SPACE_MEM_TYPE,
		PCIL0_BASE,				/* offset */
		0x0,					/* extent base */
		PCIL0_SIZE				/* extent limit */
	}
};

struct plb_dev ibm405gpr_plb_devices[] = {
	{ "cpu",   0,  -1, NULL },
	{ "ecc",   0,  16, NULL },
	{ "opb",   0,  -1, &ibm405gpr_opb_devices },
	{ "pchb",  0,  -1, &ibm405gpr_pci_config },
	{ NULL }
};

struct ibm4xx_cpudev ibm405gpr_cpu = {
	"IBM PPC405GPr",
	PVR_405GPR,
	&ibm405gpr_plb_devices,
};

struct opb_bus_space *
opb_get_bus_space_cpu(void)
{

	return &ibm405gpr_opb_bus_space;
}
